###############################################################################
#!/bin/sh                                                                     #
#                                                                             #
# auditx.sh                                                                   #
#                                                                             #
# DESC:                                                                       #
#   h4x1ng preparation script, which automatically do some basic              #
#   penetration and information gathering tests using well-known tools.       #
#   other tools will be tried after port state check from nmap logfile.       #
#   all results will be logged in a specified folder.                         #
#                                                                             #
# NOTES:                                                                      #
#   th1z 1z n0t a 1337 h4x0r t00l!!! jUs7 s4v3s s0m3 t1m3. also, it's still   #
#   not finished... no, no 0day techniques. if you need some l33t stuff       #
#   let me know, i can sell you.                                              #
#                                                                             #
# TESTED:                                                                     #
#   - gentoo (2.6.35)                                                         #
#   - debian (2.6.28)                                                         #
#   - freebsd 8.1                                                             #
#   - netbsd 5.0.2                                                            #
#                                                                             #
# TODO:                                                                       #
#   - not audit default ports for a specified service only...parse nmap file  #
#     and grep out service names                                              #
#   - new option for excluding tools/tests                                    #
#   - more tools and tests, especially for UDP                                #
#   - upgrade tools and tests, f.e.: use HTTP 1.1 methods, use more/other     #
#     options for $tool                                                       #
#                                                                             #
# GREETS:                                                                     #
#   - to my darling and son (es ces sirum em!)                                #
#   - to srm (thanks for support)                                             #
#   - to all my friends and ppl i do like                                     #
#                                                                             #
# DATE:                                                                       #
#   Mon Aug 30 21:06:00 CEST 2010                                             #
#                                                                             #
# BY:                                                                         #
#   noptrix - http://www.noptrix.net/                                         #
###############################################################################


### global ###

# auditx.sh version
VERSION="auditx.sh v0.1"

# clear
FALSE="0"
TRUE="1"

# return status codes
SUCCESS="1337"
FAILURE="31337"

# devices we need
DEV_NULL="/dev/null"
DEV_URANDOM="/dev/urandom"
DEV_STDOUT="/dev/stdout"

# verbose mode - default: OFF
VERBOSE_OFF="$DEV_NULL"
VERBOSE_ON="$DEV_STDOUT"
REDIRECT="$VERBOSE_OFF"

# CHANGE THIS! define here, where to save all results and files
auditx_DIR="$HOME/auditx/"

# after parsing hosts from nmap logfile we put them temporary in a list
HOSTS_LIST="hosts.lst"

# here we log "generic" host information
HOST_INFO_DIR="host_info"

# tcp or udp service audits? default: tcp
TCP="tcp"
UDP="udp"
PROTOCOL="$TCP"


### system utilities ### 

AWK=`whereis awk | awk '{print $2}'` # lol
MKDIR=`whereis mkdir | $AWK '{print $2}'`
RM=`whereis rm | $AWK '{print $2}'`
MV=`whereis mv | $AWK '{print $2}'`
HEAD=`whereis head | $AWK '{print $2}'`
WHOAMI=`whereis whoami | $AWK '{print $2}'`
GREP=`whereis grep | $AWK '{print $2}'`
CUT=`whereis cut | $AWK '{print $2}'`
CP=`whereis cp | $AWK '{print $2}'`
SED=`whereis sed | $AWK '{print $2}'`
CAT=`whereis cat | $AWK '{print $2}'`
TR=`whereis tr | $AWK '{print $2}'`
DATE=`whereis date | $AWK '{print $2}'`
TEE=`whereis tee | $AWK '{print $2}'`
SYSUTILS="awk mkdir rm head whoami grep cut cp sed cat tr date tee"


### net and security tools ###

# generic and host information related
NC=`whereis nc | $AWK '{print $2}'`
HOST=`whereis host | $AWK '{print $2}'`
AMAP=`whereis amap | $AWK '{print $2}'`
WHOIS=`whereis whois | $AWK '{print $2}'`
TRACEROUTE=`whereis traceroute | $AWK '{print $2}'`
TCPTRACEROUTE=`whereis tcptraceroute | $AWK '{print $2}'`
HPING=`whereis hping | $AWK '{print $2}'`
SINFP=`whereis sinfp | $AWK '{print $2}'`
OPENSSL=`whereis openssl | $AWK '{print $2}'`

# ftp related
FTPMAP=`whereis ftpmap | $AWK '{print $2}'`

# smtp related
SMTPSCAN=`whereis smtpscan | $AWK '{print $2}'`
SMTPMAP=`whereis smtpmap | $AWK '{print $2}'`

# dns related
DIG=`whereis dig | $AWK '{print $2}'`
DNSWALK=`whereis dnswalk | $AWK '{print $2}'`
DNSTRACER=`whereis dnstracer | $AWK '{print $2}'`

# finger related
FINGER_CLI=`whereis finger | $AWK '{print $2}'`

# http related
HTTPING=`whereis httping | $AWK '{print $2}'`
HTTP_PING=`whereis http_ping | $AWK '{print $2}'`
HTTPRINT=`whereis httprint | $AWK '{print $2}'`
NIKTO=`whereis nikto | $AWK '{print $2}'`

# rpc related
RPCINFO=`whereis rpcinfo | $AWK '{print $2}'`

# netbios ssn/ns samba
SMBCLIENT=`whereis smbclient | $AWK '{print $2}'`
NBAUDIT=`whereis nbaudit | $AWK '{print $2}'`
NBTSCAN=`whereis nbtscan | $AWK '{print $2}'`

# snmp related
SNMPWALK=`whereis snmpwalk | $AWK '{print $2}'`

# net and security tools list (for checks)
SECTOOLS="whois host traceroute hping nc ftpmap smtpscan smtpmap dnswalk amap 
          nikto openssl sinfp finger dnstracer httprint httping http_ping dig
          tcptraceroute rpcinfo smbclient nbaudit nbtscan snmpwalk"


### options for tools ###

# change according to your needs!
HOST_GET_NS_OPTS="-t ns"
HOST_DNS_ZT_OPTS="-l"
TRACEROUTE_OPTS="-m 15"
TCPTRACEROUTE_OPTS="-m 15"
HPING_UPTIME_OPTS="-S --tcp-timestamp -c 4 -p"
AMAP_OPTS="-bqdv -c 64"
AMAP_UDP_OPTS="-bqdv -c 64 -u"
NC_OPTS="-vv -w2"
NC_UDP_OPTS="-vvu -w2"
FTPMAP_OPTS="-s"
SMTPSCAN_OPTS="-i 3 -v"
DNSTRACER_OPTS="-c -t 3 -4"
SINFP_OPTS="-C -v"
HOST_DNS_VER_OPTS="-c chaos -t txt version.bind"
DIG_DNS_VER_OPTS="version.bind chaos txt"
FINGER_CLI_OPTS="-s root@"
NIKTO_HTTP_OPTS="-C all -h"
NIKTO_HTTPS_OPTS="-C all -p 443 -ssl -h"
HTTP_HEAD_REQ="HEAD / HTTP/1.0\r\n"
HTTP_GET_REQ="GET / HTTP/1.0\r\n"
HTTP_POST_REQ="POST / HTTP/1.0\r\n"
HTTP_OPTIONS_REQ="OPTIONS / HTTP/1.0\r\n"
HTTP_TRACE_REQ="TRACE / HTTP/1.0\r\n"
HTTP_PROPFIND_REQ="PROPFIND / HTTP/1.0\r\n"
HTTP_PUT_REQ="PUT / HTTP/1.0\r\n"
HTTPING_OPTS="-c 3 -t 6 -s -a -h"
HTTPING_SSL_OPTS="-c 3 -t 6 -s -a -l -h"
HTTP_PING_OPTS="-c 3 -i 2 http://"
HTTPRINT_OPTS="-P0 -th 16 -h"
HTTPRINT_SIGS="-s $HOME/signatures.txt"  # signatures file - CHANGE THIS!
AUTH_OPTS="version\r\n"
OPENSSL_OPTS="s_client -connect"
RPCINFO_OPTS="-p"
#SMBCLIENT_OPTS="-L"
NBAUDIT_OPTS=""
NBTSCAN_OPTS=""
SNMPWALK_OPTS="-v 1 -c public"


### port list ###

# tcp ports we want to audit
FTP="21"
SSH="22"
TELNET="23"
SMTP="25"
DNS="53"
FINGER="79"
HTTP="80"
POP3="110"
RPC="111"
AUTH="113"
EPMAP="135"
NETBIOS_NS="137"
NETBIOS_SSN="139"
IMAP="143"
HTTPS="443"
MSDS="445"
SMTPS="465"
IMAPS="993"
POP3S="995"
MYSQL="3306"

# udp ports we want to audit
DNS_UDP="53"
TFTP_UDP="69"
SNMP_UDP="161"


### logfiles ### 

WHOIS_IP_LOG="whois_ip.log"
WHOIS_DOMAIN_LOG="whois_domain.log"
TRACEROUTE_LOG="traceroute.log"
NAMESERVERS_LOG="nameservers.log"
HOST_DNS_ZT_LOG="host_dns_zonetransfer.log"
DIG_LOG="dig_zonetransfer.log"
DNSWALK_LOG="dnswalk.log"
TCPTRACEROUTE_LOG="tcptraceroute.log"
HPING_UPTIME_LOG="hping_uptime.log"
AMAP_LOG="amap_"
NC_LOG="netcat_"
FTPMAP_LOG="ftpmap.log"
SMTPSCAN_LOG="smtpscan.log"
DNSTRACER_LOG="dnstracer.log"
SINFP_LOG="sinfp.log"
HOST_DNS_VER_LOG="host_dns_version.log"
DIG_DNS_VER_LOG="dig_dns_version.log"
FINGER_LOG="finger.log"
HTTP_HEAD_LOG="http_HEAD.log"
HTTP_GET_LOG="http_GET.log"
HTTP_POST_LOG="http_POST.log"
HTTP_OPTIONS_LOG="http_OPTIONS.log"
HTTP_TRACE_LOG="http_TRACE.log"
HTTP_PROPFIND_LOG="http_PROPFIND.log"
HTTP_PUT_LOG="http_PUT.log"
HTTPING_LOG="httping.log"
HTTPING_SSL_LOG="httping_ssl.log"
HTTP_PING_LOG="http_ping.log"
HTTPRINT_LOG="httprint.log"
NIKTO_HTTP_LOG="nikto_http.log"
NIKTO_HTTPS_LOG="nikto_https.log"
AUTH_LOG="auth.log"
OPENSSL_LOG="openssl_"
RPCINFO_LOG="rpcinfo.log"
SMBCLINET_LOG="smbclient.log"
NBAUDIT_LOG="nbaudit.log"
NBTSCAN_LOG="nbtscan.log"
SNMPWALK_LOG="snmpwalk.log"


### code ###

# our leet banner
banner()
{
    echo "-----------------------------------"
    echo "auditx.sh - http://www.noptrix.net/"
    echo "-----------------------------------"

    return $SUCCESS
}


# usage and help messages
usage()
{
    echo -e "\nUSAGE: $0 -c <logfile> [options]\n"
    echo -e "REQUIRED:\n"
    echo -e "  -c <logfile>    - nmap logfile (grepable format!)\n"
    echo -e "OPTIONS:\n"
    echo "  -p <protocol>   - tcp or udp (default: tcp)"
    echo "  -l              - list supported tools"
    echo "  -v              - verbose mode"
    echo "  -h              - print help"
    echo "  -V              - print version"
    echo -e "\nNOTES:"
    echo "  - you have to be uid 0 in order to not fail on some tools and tests."
    echo "    this is for example, because of tcp raw sockets or any other
    actions, where you need root priveleges."

    return $SUCCESS
}


# test for uid
test_uid()
{
    echo "[*] user id"

    if [ `$WHOAMI` != "root" ]
    then
        echo "[-] WARNING: you are not root, some tests will fail!"
    fi

    return $SUCCESS
}


# test if $HOME variable is set
test_home_var()
{
    echo "[*] \$HOME variable"

    if [ -z $HOME ];
    then
        echo "[-] ERROR: your \$HOME is not set!"
        exit $FAILURE
    else
        return $TRUE
    fi

    return $SUCCESS
}


# is it an nmap grepable logfile?
test_nmap_file()
{
    echo "[*] nmap logfile"

    if [ -f $nmap_file ]
    then
        nmap_valid=`$HEAD -n 1 $nmap_file | $GREP -i "# Nmap" | 
        $CUT -d " " -f 1-2`
        if [ "$nmap_valid" = "# Nmap" ]
        then
            nmap_grepable=`$HEAD -n 1 $nmap_file | $GREP "\-oG"`
            if [ ! -z "$nmap_grepable" ]
            then
                return $TRUE
            else 
                echo "[-] ERROR: $nmap_file is not a grepable nmap logfile!"
                exit $FAILURE
            fi
        else
            echo "[-] ERROR: $nmap_file is not an nmap logfile!"
            exit $FAILURE
        fi
    else
        echo "[-] ERROR: $nmap_file is not a regular file!"
        exit $FAILURE
    fi

    return $SUCCESS
}


# ensure, user has supplied correct protocol
test_protocol()
{
    echo "[*] protocol"

    if [ "$PROTOCOL" != "$TCP" -a "$PROTOCOL" != "$UDP" ]
    then
        echo "[-] ERROR: unknown protocol!"
        exit $FAILURE
    fi

    return $SUCCESS
}


# run all tests defined above
run_tests()
{
    test_nmap_file
    if [ $? -eq $TRUE ]
    then
        test_home_var
        if [ $? -eq $TRUE ]
        then
            test_uid
            test_protocol
        fi
    fi

    return $SUCCESS
}


# creates auditx dir where all results will be saved and change into this dir
create_auditx_dir()
{
    echo "[*] creating auditx dir"

    if [ -d "$auditx_DIR" ]
    then
        echo "[-] ERROR: auditx directory already exists!"
        exit $FAILURE
    else
        $MKDIR $auditx_DIR > $DEV_NULL 2>&1
    fi

    $CP $nmap_file $auditx_DIR
    cd "$auditx_DIR"

    return $SUCCESS
}


# parse hosts from nmap logfile
parse_targets()
{
    echo "[*] parsing targets"

    foo=`$GREP -i "ports:" $nmap_file | $AWK '{print $2}'`
    targets=`echo "$foo" | $AWK '{gsub("", ""); print $1}'`

    return $SUCCESS
}


# create target dirs and list of hosts
create_target_dirs()
{
    echo "[*] creating target dirs"

    for i in "$targets"
    do
        $MKDIR $i > $DEV_NULL 2>&1
        echo "$i" >> $HOSTS_LIST
    done

    return $SUCCESS
}


# parse open tcp/udp ports and create a ports-list for each host
create_ports_list()
{
    echo "[*] creating ports list"

    # i know, ugly... but hey, it w0rxx! ;)
    for i in `$CAT $HOSTS_LIST`
    do
        ports=`$GREP "$i" $nmap_file | $TR -s ',: ' '\n' | $GREP '/' | 
        $SED $SED_REGEXP_OPT 's/\/open.*//g;/filtered|closed|\(|\)|\//d'`

        # now, create port lists
        echo "$ports" > "$i-ports.lst"
    done

    return $SUCCESS
}


# which sed is used? checks sed version and sets '-E' or '-r' option 
check_sed_version()
{
    echo "[*] sed version"

    $SED -h 2> "sedtest.txt"

    sedhead=`$HEAD -n 1 "sedtest.txt" | cut -d " " -f 2`
    
    if [ "$sedhead" == "invalid" ]
    then
        # GNU
        SED_REGEXP_OPT="-r"
    else
        SED_REGEXP_OPT="-E"
    fi

    `$RM -f sedtest.txt`

    return $SUCCESS
}


# check availability for system utilities
check_sysutils()
{
    echo "[*] system utilities"

    for i in $SYSUTILS
    do
        util=`whereis $i | $AWK '{print $2}'`
        if [ -z "$util" -o ! -x "$util" ]
        then
            echo "[-] ERROR: \"$i\" doesn't exist"
            exit $FAILURE
        fi
    done

    check_sed_version

    return $SUCCESS
}


# check availability for net and security tools
check_sectools()
{
    echo "[*] net and security tools"

    for i in $SECTOOLS
    do
        tool=`whereis $i | $AWK '{print $2}'`
        if [ -z "$tool" -o ! -x "$tool" ]
        then
            echo "[-] WARNING: \"$i\" doesn't exist"
        fi
    done

    return $SUCCESS
}


# get only the domain, not hostnames or subdomains
get_domain_names()
{
    # get name from nmap logfile first
    hostname=`$GREP -vE "#|Status:" $auditx_DIR/$nmap_file | 
    $GREP "$target_ipaddr" | $CUT -d " " -f 3 | 
    $SED $SED_REGEXP_OPT 's/:|\(|\)|Ports|done//g'`

    # we are only interested in domain-name!
    target_domain=`echo $hostname | $GREP "\\\\." | 
    $AWK '/[a-zA-Z]/ {num=split($0, a, "."); print a[num-1] "." a[num];}'`

    return $SUCCESS
}


# should be clear 
get_nameservers()
{
    # get nameservers from $target_domain or from $target_ipaddr
    if [ ! -z "$target_domain" ]
    then
        nameservers=`$HOST $HOST_GET_NS_OPTS $target_domain | $CUT -d " " -f 4|
        $SED 's/\.$//g'`
    else
        nameservers=`$HOST $HOST_GET_NS_OPTS $target_ipaddr | 
        $GREP "name pointer" | $CUT -d " " -f 5 | $SED 's/\.$//g'`
    fi


    # log all found nameservers
    for n in "$nameservers"
    do
        echo "$n" >> $NAMESERVERS_LOG 2>&1
    done

    return $SUCCESS
}


# whois against ip-address and domain
audit_whois()
{
    echo -e "\n[+] WHOIS"

    # whois against ip-address
    echo "[*] ip-address"
    $WHOIS $target_ipaddr | $TEE $WHOIS_IP_LOG > $REDIRECT 2>&1

    # whois against domain-name
    if [ ! -z "$target_domain" ]
    then
        echo "[*] domain"
        $WHOIS $target_domain | $TEE $WHOIS_DOMAIN_LOG > $REDIRECT 2>&1
    fi

    return $SUCCESS
}


# test for dns zonetransfers
audit_zonetransfer()
{
    echo -e "\n[+] DNS ZONETRANSFER"

    echo "[*] host"
    echo "[*] dig"
    
    counter=0
    
    for n in `$CAT $NAMESERVERS_LOG`
    do
        counter=`expr $counter + 1`
        if [ ! -z "$target_domain" ]
        then
            $HOST $HOST_DNS_ZT_OPTS $target_domain $n 2>&1 | 
            $TEE "$HOST_DNS_ZT_LOG.$counter" > $REDIRECT
            $DIG "@$n" $target_domain "axfr" 2>&1 | 
            $TEE "$DIG_LOG.$counter" > $REDIRECT
        fi
    done

    if [ ! -z "$target_domain" ]
    then
        echo "[*] dnswalk"
        $DNSWALK "$target_domain." 2>&1 | $TEE $DNSWALK_LOG > $REDIRECT
    fi

    return $SUCCESS
}   


# get routes! traceroute etc.
audit_route()
{
    echo -e "\n[+] ROUTE"

    echo "[*] traceroute"
    $TRACEROUTE $TRACEROUTE_OPTS $target_ipaddr 2>&1 | 
    $TEE $TRACEROUTE_LOG > $REDIRECT

    echo "[*] tcptraceroute"
    $TCPTRACEROUTE $TCPTRACEROUTE_OPTS $target_ipaddr 2>&1 |
    $TEE $TCPTRACEROUTE_LOG > $REDIRECT

    return $SUCCESS
}


# performs uptime tests
audit_uptime()
{
    echo -e "\n[+] UPTIME"
    
    # find first open port - this is ok for hping 
    port=`$HEAD -n 1 $auditx_DIR$target_ipaddr-ports.lst`

    echo "[*] hping"
    $HPING $HPING_UPTIME_OPTS $port $target_ipaddr 2>&1 | 
    $TEE $HPING_UPTIME_LOG > $REDIRECT

    return $SUCCESS
}


# trace dns servers of targets
trace_dns()
{
    echo -e "\n[+] DNS TRACING"

    if [ ! -z "$target_domain" ]
    then
        echo "[*] dnstracer"
        $DNSTRACER $DNSTRACER_OPTS $target_domain 2>&1 |
        $TEE $DNSTRACER_LOG > $REDIRECT
    fi

    return $SUCCESS
}


# do active os fingerprinting against our targets
os_fingerprint()
{
    echo -e "\n[+] OS FINGERPRINT"

    # find first open port
    port=`$HEAD -n 1 $auditx_DIR$target_ipaddr-ports.lst`

    echo "[*] sinfp"
    $SINFP $SINFP_OPTS -i $target_ipaddr -p $port 2>&1 |
    $TEE $SINFP_LOG > $REDIRECT

    echo "[*] nmap"

    return $SUCCESS
}


# host information gathering for ip-address, domain, route, uptime etc.
get_host_infos()
{
    get_domain_names
    get_nameservers
    audit_whois
    audit_zonetransfer
    audit_route
    audit_uptime
    trace_dns
    os_fingerprint

    return $SUCCESS
}


# run generic tcp port tests
audit_generic_tcp()
{
    echo "[*] amap"
    $AMAP $AMAP_OPTS $target_ipaddr $port 2>&1 |
    $TEE $AMAP_LOG$port"_tcp.log" > $REDIRECT

    # send random 512-bytes to destination port
    echo "[*] netcat"
    $HEAD -c 512 $DEV_URANDOM | $NC $NC_OPTS $target_ipaddr $port 2>&1 | 
    $TEE $NC_LOG$port"_tcp.log" > $REDIRECT

    return $SUCCESS
}


# run generic udp port tests
audit_generic_udp()
{
    echo "[*] amap"
    $AMAP $AMAP_OPTS $target_ipaddr $port 2>&1 |
    $TEE $AMAP_LOG$port"_udp.log" > $REDIRECT
    
    # send random 512-bytes to destination port
    echo "[*] netcat"
    $HEAD -c 512 $DEV_URANDOM | $NC $NC_UDP_OPTS $target_ipaddr $port 2>&1 |
    $TEE $NC_LOG$_port"_udp.log" > $REDIRECT

    return $SUCCESS
}


# run ftp tools and tests
audit_ftp()
{
    echo -e "\n[+] FTP"
    
    audit_generic_tcp

    echo "[*] ftpmap"
    $FTPMAP $FTPMAP_OPTS $target_ipaddr 2>&1 | $TEE $FTPMAP_LOG > $REDIRECT

    return $SUCCESS
}


# run ssh tools and tests
audit_ssh()
{
    echo -e "\n[+] SSH"

    audit_generic_tcp

    return $SUCCESS
}


# run telnet tools and tests
audit_telnet()
{
    echo -e "\n[+] TELNET"

    audit_generic_tcp

    return $SUCCESS
}


# run smtps tools and tests
audit_smtps()
{
    echo -e "\n[+] SMTPS"

    audit_generic_tcp

    return $SUCCESS
}


# run smtp tools and tests
audit_smtp()
{
    if [ "$port" -eq "$SMTPS" ]
    then
        audit_smtps
    else 
        echo -e "\n[+] SMTP"
        audit_generic_tcp

        echo "[*] smtpscan"
        $SMTPSCAN $SMTPSCAN_OPTS $target_ipaddr 2>&1 |
        $TEE $SMTPSCAN_LOG > $REDIRECT
    fi

    return $SUCCESS
}


# run dns tools and tests
audit_dns()
{
    echo -e "\n[+] DNS"

    audit_generic_tcp

    echo "[*] host"
    $HOST $HOST_DNS_VER_OPTS $target_ipaddr 2>&1 |
    $TEE $HOST_DNS_VER_LOG > $REDIRECT

    echo "[*] dig"
    $DIG "@$target_ipaddr" $DIG_DNS_VER_OPTS 2>&1 |
    $TEE $DIG_DNS_VER_LOG > $REDIRECT

    return $SUCCESS
}


# tftp tools and tests
audit_tftp()
{
    echo -e "\n[+] TFTP"

    audit_generic_udp

    return $SUCCESS
}

# run finger tools and tests
audit_finger()
{
    echo -e "\n[+] FINGER"

    audit_generic_tcp

    echo "[*] finger"
    $FINGER_CLI $FINGER_CLI_OPTS$target_ipaddr 2>&1 |
    $TEE $FINGER_LOG > $REDIRECT

    return $SUCCESS
}


# send HEAD, GET, POST etc. http-requests
send_http_requests()
{
    echo "[*] head"
    echo -e $HTTP_HEAD_REQ | $NC $target_ipaddr $port 2>&1 |
    $TEE $HTTP_HEAD_LOG > $REDIRECT

    echo "[*] get"
    echo -e $HTTP_GET_REQ | $NC $target_ipaddr $port 2>&1 |
    $TEE $HTTP_GET_LOG > $REDIRECT

    echo "[*] post"
    echo -e $HTTP_POST_REQ | $NC $target_ipaddr $port 2>&1 |
    $TEE $HTTP_POST_LOG > $REDIRECT

    echo "[*] options"
    echo -e $HTTP_OPTIONS_REQ | $NC $target_ipaddr $port 2>&1 |
    $TEE $HTTP_OPTIONS_LOG > $REDIRECT

    echo "[*] trace"
    echo -e $HTTP_TRACE_REQ | $NC $target_ipaddr $port 2>&1 |
    $TEE $HTTP_TRACE_LOG > $REDIRECT

    echo "[*] propfind"
    echo -e $HTTP_PROPFIND_REQ | $NC $target_ipaddr $port 2>&1 |
    $TEE $HTTP_PROPFIND_LOG > $REDIRECT

    echo "[*] put"
    echo -e $HTTP_PUT_REQ | $NC $target_ipaddr $port 2>&1 |
    $TEE $HTTP_PUT_LOG > $REDIRECT

    return 1337
}


# run http tools and tests
audit_http()
{
    if [ "$port" -eq "$HTTPS" ]
    then
        audit_https
    else
        echo -e "\n[+] HTTP"
        audit_generic_tcp
        send_http_requests

        echo "[*] httping"
        $HTTPING $HTTPING_OPTS $target_ipaddr 2>&1 |
        $TEE $HTTPING_LOG > $REDIRECT

        echo "[*] http_ping"
        $HTTP_PING $HTTP_PING_OPTS$target_ipaddr 2>&1 |
        $TEE $HTTP_PING_LOG > $REDIRECT

        echo "[*] httprint"
        $HTTPRINT $HTTPRINT_OPTS $target_ipaddr $HTTPRINT_SIGS 2>&1 |
        $TEE $HTTPRINT_LOG > $REDIRECT

        echo "[*] nikto"
        $NIKTO $NIKTO_HTTP_OPTS $target_ipaddr 2>&1 |
        $TEE $NIKTO_HTTP_LOG > $REDIRECT
    fi

    return $SUCCESS
}


# run pop3s tools and tests
audit_pop3s()
{
    echo -e "\n[+] POP3S"

    audit_generic_tcp

    return $SUCCESS
}

# run pop3 tools and tests
audit_pop3()
{
    if [ "$port" -eq "$POP3S" ]
    then
        audit_pop3s
    else
        echo -e "\n[+] POP3"
        audit_generic_tcp
    fi

    return $SUCCESS
}


# run rpc tools and tests
audit_rpc()
{
    echo -e "\n[+] RPC"

    audit_generic_tcp

    echo "[*] rpcinfo"
    $RPCINFO $RPCINFO_OPTS $target_ipaddr 2>&1 | $TEE $RPCINFO_LOG > $REDIRECT

    return $SUCCESS
}


# run auth tools and tests
audit_auth()
{
    echo -e "\n[+] AUTH"

    audit_generic_tcp

    # try getting authd version
    echo "[*] version"
    echo -e $AUTH_OPTS | $NC $NC_OPTS $target_ipaddr $port 2>&1 |
    $TEE $AUTH_LOG > $REDIRECT


    return $SUCCESS
}


# run imaps tools and tests
audit_imaps()
{
    echo -e "\n[+] IMAPS"

    audit_generic_tcp

    return $SUCCESS
}


# run imap tools and tests
audit_imap()
{
    if [ "$port" -eq "$IMAPS" ]
    then
        audit_imaps
    else
        echo -e "\n[+] IMAP"
        audit_generic_tcp
    fi

    return $SUCCESS
}


# run netbios tools and tests
audit_netbios()
{
    echo -e "\n[+] NETBIOS"

    audit_generic_tcp

    echo "[*] nbaudit"
    $NBAUDIT $NBAUDIT_OPTS $target_ipaddr 2>&1 | $TEE $NBAUDIT_LOG > $REDIRECT

    echo "[*] nbtscan"
    $NBTSCAN $NBTSCAN_OPTS $target_ipaddr 2>&1 | $TEE $NBTSCAN_LOG > $REDIRECT

    #echo "[*] smbclient"
    #$SMBCLIENT $SMBCLIENT_OPTS $target_ipaddr 2>&1 | 
    #$TEE $SMBCLIENT_LOG > $REDIRECT

    return $SUCCESS
}


# run snmp tools and tests
audit_snmp()
{
    echo -e "\n[+] SNMP"

    audit_generic_tcp
    audit_generic_udp

    echo "[*] snmpwalk"
    $SNMPWALK $SNMPWALK_OPTS $target_ipaddr 2>&1 | 
    $TEE $SNMPWALK_LOG > $REDIRECT

    return $SUCCESS
}


# run https tools and tests
audit_https()
{
    echo -e "\n[+] HTTPS"

    audit_generic_tcp

    echo "[*] httping"
    $HTTPING $HTTPING_SSL_OPTS $target_ipaddr 2>&1 |
    $TEE $HTTPING_SSL_LOG > $REDIRECT

    # send 512 random bytes to ssl-port
    echo "[*] openssl"
    $HEAD -c 512 $DEV_URANDOM | 
    $OPENSSL $OPENSSL_OPTS "$target_ipaddr:$port" 2>&1 |
    $TEE "$OPENSSL_LOG$port.log" > $REDIRECT

    echo "[*] nikto"
    $NIKTO $NIKTO_HTTPS_OPTS $target_ipaddr 2>&1 |
    $TEE $NIKTO_HTTPS_LOG > $REDIRECT

    return $SUCCESS
}


# run mysql tools and tests
audit_mysql()
{
    echo -e "\n[+] MYSQL"

    audit_generic_tcp
    
    return $SUCCESS
}


# select tools and start tcp related audits
start_tcp_audits()
{
    for j in `$CAT $auditx_DIR$i-ports.lst`
    do
        port=$j
        $MKDIR $port"_tcp"
        cd $port"_tcp"
        
        case "$j" in
            $FTP) audit_ftp ;;
            $SSH) audit_ssh ;;
            $TELNET) audit_telnet ;;
            $SMTP) audit_smtp ;;
            $DNS) audit_dns ;;
            $FINGER) audit_finger ;;
            $HTTP) audit_http ;;
            $RPC) audit_rpc ;;
            $POP3) audit_pop3 ;;
            $AUTH) audit_auth ;;
            $IMAP) audit_imap ;;
            $EPMAP) audit_netbios ;;
            $NETBIOS_NS) audit_netbios ;;
            $NETBIOS_SSN) audit_netbios ;;
            $SNMP) audit_snmp ;;
            $HTTPS) audit_http ;;
            $MSDS) audit_netbios ;;
            $SMTPS) audit_smtp ;;
            $IMAPS) audit_imap ;;
            $POP3S) audit_pop3 ;;
            $MYSQL) audit_mysql ;;
            *)
                echo -e "\n[+] PORT $port (TCP)"
                audit_generic_tcp ;;
        esac
        cd "../"
    done

    return $SUCCESS
}


# select tools and start udp related audits
start_udp_audits()
{
    for j in `$CAT $auditx_DIR$i-ports.lst`
    do
        port=$j
        $MKDIR $port"_udp"
        cd $port"_udp"

        case "$j" in
            $DNS_UDP) audit_dns ;;
            $TFTP_UDP) audit_tftp ;;
            $SNMP_UDP) audit_snmp ;;
            *)
                echo -e "\n[+] PORT $port (UDP)"
                audit_generic_udp ;;
        esac
        cd "../"
    done

    return $SUCCESS
}


# start footprinting related tools and tests
start_footprint()
{
    $MKDIR $HOST_INFO_DIR
    cd $HOST_INFO_DIR
    get_host_infos

    # we are done, so we can cd back
    cd "../"

    return $SUCCESS
}


# start security tools and tests
start_audits()
{
    for i in `$CAT $HOSTS_LIST`
    do
        # change to current target dir, we are going to attack
        cd "$i"

        # we need ip-address
        target_ipaddr=$i
        echo -e "\n\n[+] HOST ATTACK ($i)"

        start_footprint
        
        if [ $PROTOCOL == $TCP ]
        then
            start_tcp_audits
        else
            start_udp_audits
        fi
        
        cd "$auditx_DIR"
    done

    return $SUCCESS
}


# clean all temporary created files and mv auditx/ to auditx-<DATE>
clean_up()
{
    echo -e "\n[+] GAME OVER"

    # build date format
    foo="auditx-"
    bar=`$DATE | $CUT -d " " -f 1-2,3,6 | $SED -e 's/ /-/g'`
    foobar=$foo$bar

    echo "[*] cleaning files"
    cd "$auditx_DIR"
    `$RM -rf *.lst $nmap_file`
    cd ".."
    `$MV "auditx" $foobar` > $DEV_NULL 2>&1

    return $SUCCESS
}


# list all supported net and security tools
list_net_sec_tools()
{
    banner
    
    echo "[+] supported net and security tools"

    for i in $SECTOOLS
    do
        echo "[*] $i"
    done

    return $SUCCESS
}


# just ensure if '-c' option along with nmap logfile was chosen
ensure_nmap_logfile_option()
{
    if [ -z $nmap_file ]
    then
        echo "[-] ERROR: WTF? mount /dev/brain!"
        exit 1337
    fi

    return $SUCCESS
}


# parse options, set flags, run auditx tests
control_options()
{
    args=`getopt c:p:x:lvhV $*`

    for i
    do
        case "$i" in
            -c)
                shift
                nmap_file=$1
                banner
                ;;
            -p)
                shift
                PROTOCOL=$2
                ;;
            -x)
                ;;
            -l)
                list_net_sec_tools
                exit $SUCCESS
                ;;
            -v)
                REDIRECT=$VERBOSE_ON
                ;;
            -V)
                echo $VERSION
                exit $SUCCESS
                ;;
            -h)
                banner
                usage
                exit $SUCCESS
                ;;
        esac
    done 
    
    ensure_nmap_logfile_option

    return $TRUE
}


# prepare everything before starting any audits and tests
prepare_audits()
{
    echo -e "\n[+] CHECKS"
    run_tests
    check_sysutils
    check_sectools
    
    echo -e "\n[+] DIRS AND FILES"
    create_auditx_dir
    parse_targets
    create_target_dirs
    create_ports_list

    return $SUCCESS
}


# let's go...
main()
{
    if [ $# -lt 1 ]
    then
        echo "[-] WARNING: -h for help and usage!"
        exit $FAILURE
    else
        control_options $*
        if [ $? -eq $TRUE ]
        then
            prepare_audits
            start_audits
            clean_up
        fi
    fi
    
    echo "[*] we are done. hope it helped!"

    return $SUCCESS
}

# o/ HELLO LAMAZ, we start here!
main $*

# EOF
